/*
# MIT License
#
# Copyright (c) 2020 UCLouvain
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.*/


#include <stdio.h>
#include <stdlib.h>
#include "verilated.h"
#include "include_modules.h"

#define MAX_ENCRYPT 10
#define MAX_CYCLES 100000
#define MAXLEN 1000
#ifndef D
#define D 2
#endif
#ifdef __cplusplus
extern "C" {
    void sim_run_probe(uint32_t *inputs, uint32_t ilen,
            uint32_t *cycles, uint32_t *probes,uint32_t n_cycles);
}
#endif
void tick(Vtopmodule *tb){
    tb->clk = 0;
    tb->eval();
    tb->clk = 1;
    tb->eval();
    tb->clk = 0;
    tb->eval();
}


void sim_run_probe(uint32_t *inputs, uint32_t ilen,
        uint32_t *cycles, uint32_t *probes,uint32_t n_cycles){
    long long cycle_counter = 0;
    size_t i,c;
    i=0;
    c=0;

    Vtopmodule *tb = new Vtopmodule;
    tb->clk = 0;
    tb->rst = 1;
    tick(tb);
    tick(tb);
    tb->rst = 0;
    tick(tb);
    tick(tb);
    tb->ready_bus_out = 1;
    tick(tb);

    for(cycle_counter=0;cycle_counter<MAX_CYCLES;cycle_counter++){
        // send inputs
        if(i<ilen){ // there is still some data to send
            if(tb->ready_bus_in){ // can send data
                tb->bus_in= inputs[i];i++;
                tb->bus_in_valid = 1;
            }
        }else{
            tb->bus_in_valid = 0;
        }

        if(cycles[c]==cycle_counter){
            memcpy(&probes[c*(D)*4],&tb->spook_MSK->datapath_core->clyde_core->clyde_core->sharing_bundle_state,D*sizeof(uint32_t)*4);
            c++;
            if(c>=n_cycles)
                break;
        }

        if(tb->bus_out_valid && tb->bus_out_last && i==ilen){
            break;
        }
        tick(tb);
    }
    delete(tb);
}

