# Demo for key byte recovery against software Spook
Next, we detail an byte recovery attack against the 3 shares software implementation of Spook. To do so, we make use of MCP-DPA on the output of the Sbox.

## Sensitive value description
Clyde is a bitslice cipher. Its state is composed of 4 32-bit words. The Sbox is on 4-bit and takes as input one bit in each of the words. This first operations performed during a Clyde execution are the following:

```
// key initialization
k = [0xdeadbe01, 0xffffff02, 0x01020303, 0x00000000]

// plaintext initialization
p = [0xdefeca7e, 0xdefeca7e, 0x11111111, 0x12345678]

// Key addition
x = k ^ p

// Sbox layer
y[1] = (x[0] & x[1]) ^ x[2]
y[0] = (x[3] & x[0]) ^ x[1]
y[3] = (y[1] & x[3]) ^ x[0]
y[2] = (y[0] & y[1]) ^ x[3]
```

In this demonstration, we target 8-bit of key which are at the input of two Sboxes. Therefore, we will recover the 2 bits in each of the key words. These are chosen to have a good SNR and are the bits 15 and 16.  The sensible value to target are the similar bits in y. This is next denoted as the sensible value s.
(For simplicity, we omit here the addition of the tweak key which is a zero string in the first Clyde call within Spook.)

## Key recovery methodology
Next, we detail the 2 stages of the attack. The corresponding scripts are [attack_mcp_dpa.py](attack_mcp_dpa.py) and [profile_mcp_dpa.py](profile_mcp_dpa.py)

## Profiling
In order to recover a key byte, we first find the location of all the shares. Given the input key shares, the nonces, a prng seed, their value can be obtained by running
```
msk_state = clyde128_encrypt_masked(nonces.T,np.zeros((4,nt),dtype=np.uint32),msk_key.T,seeds.T,Nr=1,step=0)
```
from [spook_masked.py](../interface/spook_masked.py). Then, the SNR is computed for all the shares as depicted in the following figure. The shares associated to the same variable are plot in the same color.

![Profiling Output](.figs/snr.png)

In order to perform the MCP-DPA, the leakage of each shares associated to the same 2 bits are multiplied together. Then, the four obtained multiplications are added together. The result is then 
used as the input of a profiled CPA. Namely, the mean of the result is computed for each possible 8-bit sbox output.

## Attack
Afterwards, these profiles can be used to perform the MCP-DPA. Namely, the same points as in the profiling are multiplied and summed together. For each key candidate, the expected model is correlated with the actual leakage.
