# Stella: library for side-channel leakage analysis
#
# Copyright (C) 2019,2020 UCLouvain
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# This program comes with ABSOLUTELY NO WARRANTY;
import numpy as np

class Accumulator():

    def __init__(self,l,Ns,dtype=np.int16):
        """
            This object stores the fitted inputs withing and array

            l: number of lines in the array
            Ns: number of columns
            dtype: data type of the numpy array
        """

        self._array = np.zeros((l,Ns),dtype=dtype)
        self._Ns = Ns
        self._l = l
        self._i = 0
    def fit(self,array):
        """
            fill the owned array with this one
        """
        lx,ly = array.shape
        if ly != self._Ns:
            raise Exception("Does not have the correct shape")
        m = self._i + lx
        if m > self._l:
            raise Exception("Too much data fitted")

        self._array[self._i:m,:] = array
        self._i = m

    def get(self):
        return self._array[:self._i]
